﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using UnityEngine.SceneManagement;
using UnityEngine.UI;

public class GameManager : MonoBehaviour
{
	[SerializeField] public GameObject ideaPrefab;
	[SerializeField] public GameObject player;

	[SerializeField] private int _initTime;
	[SerializeField] private Image _timeBar;
	[SerializeField] private Image _gameProgressBar;
	[SerializeField] private List<Sprite> _ideaImages;

	[SerializeField] public WaveSetting waveTuner;

	private float _remainingTime;
	//private float _gameSpeed;
	//private float _gameProgress;
	private float _gameMaxPoints;

	private bool _isWin = false;
	private bool _ended = false;

	private static GameManager _instance;
	public static GameManager instance
	{
		get
		{
			return _instance ?? null;
		}
	}

	void Awake()
	{
		_instance = this;
		DontDestroyOnLoad(this);
	}

	void Start ()
	{
		_remainingTime = _initTime;
		_gameMaxPoints = _initTime * 2.5f;
		//_gameSpeed = 5;
		//_gameProgress = 0;
	}
	
	void Update ()
	{
		if (_ended)
			return;

		// Decrease GGJ time
		_remainingTime -= Time.deltaTime;

		if (_remainingTime <= 0)
		{
			FinishGame();
		}

		_timeBar.fillAmount = _remainingTime / (float)_initTime;
		
		// Increase game progress
		//_gameProgress += Time.deltaTime * _gameSpeed;

		_gameProgressBar.fillAmount = player.GetComponent<CharacterComponent>().GetTeam().gameProgress / _gameMaxPoints;
	}

	public Sprite GetIdeaImage()
	{
		return _ideaImages[UnityEngine.Random.Range(0, _ideaImages.Count)];
	}

	private void FinishGame()
	{
		_ended = true;

		var teams = GameObject.FindObjectsOfType<Teampoint>();
		float maxPoints = 0;
		Teampoint bestTeam = null;
		foreach (var team in teams)
		{
			if (team.gameProgress > maxPoints)
			{
				maxPoints = team.gameProgress;
				bestTeam = team;
			}
		}
		
		_isWin = (bestTeam == player.GetComponent<CharacterComponent>().GetTeam());
		
		SceneManager.LoadScene("FinalScene");
	}

	public void ShowFinalResults(GameObject winner, GameObject loser)
	{
		if (_isWin)
		{
			winner.SetActive(true);
		}
		else
		{
			loser.SetActive(true);
		}
	}

	public void DestroyGameSession()
	{
		_instance = null;
		Destroy(this);
		Destroy(this.gameObject);
	}

}
