﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using UnityEngine;


public class Idea : MonoBehaviour
{
	[SerializeField] private GameObject _cloud;
	[SerializeField] private Transform _bigPoint;
	[SerializeField] private Transform _mediumPoint;
	[SerializeField] private Transform _smallPoint;
	[SerializeField] private SpriteRenderer _ideaImage;

	public bool isHided
	{
		get { return _isHided; }
		set { _isHided = value; }
	}

	private Transform _owner;
	private Vector3 _offset;
	private bool _isHided;
	private int _hiding = 0;

	private static float _offsetDistance = 0.9f;//1.4f;

	private static float offsetDistance
	{
		get { return _offsetDistance + UnityEngine.Random.value * (_offsetDistance / 4); }
	}

	public static Idea Create(GameObject prefab, Sprite ideaImage, Transform parent, Vector2 direction, bool hiding)
	{
		var idea = Instantiate(prefab).GetComponent<Idea>();
		idea._ideaImage.sprite = ideaImage;
		idea.ChangeIdeaParent(parent, direction);
		idea._isHided = hiding;
		if (hiding)
			idea._cloud.SetActive(false);

		return idea;
	}

	void Start()
	{
		
	}

	void Update()
	{
		
	}

	public void ChangeIdeaParent(Transform newParent, Vector2 direction)
	{
		//Debug.Log("Changing parent");
		if (_owner == newParent)
			return;

		if (_owner)
			_owner.GetComponent<IdeaHolderComponent>().RemoveIdea(this);

		_owner = newParent;

		_isHided = false;
		_hiding++;
		_cloud.SetActive(true);

		var dir = direction.normalized;
		transform.SetParent(newParent);
		_offset = new Vector3(dir.x * offsetDistance, dir.y * offsetDistance, 0);
		transform.localPosition = _offset;

		var cloudBorder = 0.4f;
		var distance = 0.5f;//0.8f;
		var offsetPoints = offsetDistance - cloudBorder - distance;

		//_bigPoint.localPosition = new Vector3(-dir.x * (offsetPoints / 3 + cloudBorder), -dir.y * (offsetPoints / 3  + cloudBorder));
		//_mediumPoint.localPosition = new Vector3(-dir.x * (2 * offsetPoints / 3  + cloudBorder), -dir.y * (2 * offsetPoints / 3 + cloudBorder));
		//_smallPoint.localPosition = new Vector3(-dir.x * (3 * offsetPoints / 3  + cloudBorder), -dir.y * (3 * offsetPoints / 3 + cloudBorder));
		_bigPoint.gameObject.SetActive(false);
		_mediumPoint.gameObject.SetActive(false);
		_smallPoint.gameObject.SetActive(false);
	}

	private void SetOffset()
	{
		
	}

	public void SetHiding(bool value)
	{
		//if (_isHided == value)
		//	return;

		_hiding++;
		StartCoroutine(SetHiding(_hiding, value,
			value
			? UnityEngine.Random.Range(0.1f, 1.1f)
			: UnityEngine.Random.Range(1.5f, 5.2f)));
	}

	private IEnumerator SetHiding(int hidValue, bool hide, float ds)
	{
		yield return new WaitForSeconds(ds);

		if (_hiding == hidValue)
		{
			_isHided = hide;
			_cloud.SetActive(!hide);
		}
	}

	public Vector3 GetOwnerPos()
	{
		return _owner.position;
	}

    public bool CanSteel(CharacterComponent steeler)
    {
        var teampoint = _owner.GetComponent<Teampoint>();

	    if (teampoint == null)
		    return false;

        if (teampoint.teamGD)
        {
            if (teampoint.teamGD == steeler)
            {
                return false;
            }
        }
        return true;
    }

    public bool IsItMyIdea(CharacterComponent character)
    {
        //Debug.Log(_owner.GetInstanceID());
        //Debug.Log(character.transform.GetInstanceID());
        //Debug.Log(_owner == character.transform);
        return _owner == character.transform;
    }

}

