﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;

public class NavMaster : MonoBehaviour
{

	private List<NavPoint> _points;

	private static NavMaster _instance;
	public static NavMaster instance {
		get {
			return _instance ?? null;
		}
	}

	void Awake()
	{
		_instance = this;
		//DontDestroyOnLoad(this);
	}

	void Start()
	{
		_points = new List<NavPoint>();
		var points = GameObject.FindObjectsOfType(typeof(NavPoint)) as NavPoint[];
		//Debug.Log(points.Length);
		if (points == null)
			return;

		foreach (var point in points)
		{
			if (point)
				_points.Add(point);
		}
		//Debug.Log(_points.Count);

	}

	public NavPoint GetRandomPoint()
	{
		return _points[UnityEngine.Random.Range(0, _points.Count)];
	}

	public List<NavPoint> GetPathFromTo(NavPoint start, NavPoint destination)
	{
		var path = new List<NavPoint>() {start};

		if (start == destination)
			return path;

		SearchLevel(destination, ref path);
		return path;
	}

	private bool SearchLevel(NavPoint destination, ref List<NavPoint> list)
	{
		if (list.Count == 0)
			return false;

		//Debug.Log(list.Count);

		var curPoint = list.Last();
		//list.pop_front();
		var neighbors = curPoint._nearest;
		var allWeGet = list.ToList();
		neighbors = neighbors.Where(n => allWeGet.FirstOrDefault(x => x == n) == null).ToList();

		var nearNeighbor = curPoint.PopBestWayPointTo(destination, ref neighbors);
		while (nearNeighbor != null)
		{
			list.Add(nearNeighbor);
			if (nearNeighbor == destination)
				return true;

			if (SearchLevel(destination, ref list))
				return true;

			list.Remove(nearNeighbor);

			nearNeighbor = curPoint.PopBestWayPointTo(destination, ref neighbors);
		}
		
		list.Remove(curPoint);

		return false;
	}

	public class PathNode
	{
		public NavPoint curPoint;
		public PathNode nextNode;
	}

}
