﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public class NavPoint : MonoBehaviour
{
	[SerializeField] private bool _lowDistance;
	private static float minDistance = 1.5f;

	public List<NavPoint> _nearest { get; private set; }

	void Start()
	{
		_nearest = new List<NavPoint>();
		var navPoints = GameObject.FindObjectsOfType<NavPoint>();
		foreach (var navPoint in navPoints)
		{
			if (navPoint == this)
				continue;

			var dist = minDistance;
			if (_lowDistance)
				dist -= 0.49f;

			if (Vector2.Distance(navPoint.transform.position, this.transform.position) < dist)
				_nearest.Add(navPoint);
		}
		//Debug.Log(_nearest.Count);
	}

	// redundant!
	public NavPoint GetBestWayPointTo(NavPoint destination, Transform character, out float angle)
	{
		Debug.Log("REDUNDANT");
		NavPoint bestPoint = _nearest[0];
		float minDistance = 100f;
		foreach (var navPoint in _nearest)
		{
			var dist = Vector2.Distance(destination.transform.position, navPoint.transform.position);
			if (dist < minDistance)
			{
				minDistance = dist;
				bestPoint = navPoint;
			}
		}

		angle = GetAngleFromToPoint(character.position, bestPoint.transform.position);

		return bestPoint;
	}

	public NavPoint PopBestWayPointTo(NavPoint destination, ref List<NavPoint> list)
	{
		if (list.Count == 0)
			return null;

		NavPoint bestPoint = _nearest[0];
		float minDistance = 100f;
		foreach (var navPoint in list)
		{
			var dist = Vector2.Distance(destination.transform.position, navPoint.transform.position);
			if (dist < minDistance)
			{
				minDistance = dist;
				bestPoint = navPoint;
			}
		}
		list.Remove(bestPoint);

		return bestPoint;
	}

	private float GetAngleFromToPoint(Vector3 source, Vector3 point)
	{
		return Mathf.Rad2Deg * Mathf.Atan2(point.y - source.y, point.x - source.x);
	}

}
