﻿using System;
using System.Collections;
using System.Threading;
using UnityEngine;

public class CharacterComponent : MonoBehaviour
{
    [SerializeField]
	private CharacterRenderComponent _render;
	[SerializeField]
	private Teampoint _myTeam;

	private IdeaHolderComponent _ideaHolder;

	private int _animationRatio;
    

	public static CharacterComponent Create(GameObject go)
	{
		if (!go.CheckComponents(typeof(MovementComponent), typeof(CharacterRenderComponent)))
		{
			Debug.LogError("Tried to add Character component without having AliveComponent, MovementComponent and CharacterRenderer. Reject");
			return null;
		}

		var ch = go.AddComponent<CharacterComponent>();
		go.AddComponent<BoxCollider2D>(); // TODO: change box collider to fit sprite
		var rigid = go.AddComponent<Rigidbody2D>();
		rigid.gravityScale = 0;
		rigid.freezeRotation = true;

		//ch._eyeRotation = new RotationalParameter(0, 360, 0);
		//ch._movement = go.GetComponent<MovementComponent>();
		ch._ideaHolder = go.AddComponent<IdeaHolderComponent>();

		ch._render = go.GetComponent<CharacterRenderComponent>();

		return ch;
	}

	protected void Awake()
	{
        _ideaHolder = gameObject.GetComponent<IdeaHolderComponent>();
    }

	protected void Start()
	{
		
	}

	protected void Update()
	{
		_render
			.SetAnimRatio(GetComponent<MovementComponent>()
				.direction.magnitude < 0.1f 
				? 0 
				: 1
				, true);
	}


	public void SteelIdea(Idea idea)
	{
		_ideaHolder.AddIdea(idea);
	}

	public void SetTurns(sbyte movementTurn, sbyte eyeTurn)
	{
		// MAYBE USE FOR BACK-FRONT 
	}

    public Teampoint GetTeam()
    {
        return _myTeam;
    }

    public void SetTeam(Teampoint team)
    {
        _myTeam = team;
    }

}