﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using UnityEngine;


public class CharacterRenderComponent : SimpleRenderComponent
{
    [SerializeField] private Transform _parent;
    [SerializeField] private Sprite _sprite;

	private float _playbackRatio;
	private float _animTime;

	public static CharacterRenderComponent Create(GameObject go, Sprite sprite)
	{
        var ren = go.AddComponent<CharacterRenderComponent>();
	    ren._parent = go.transform;
	    ren._sprite = sprite;

	    return ren;
	}

    void Start()
    {
        var pivot = new GameObject();
        pivot.transform.SetParent(_parent);

        var body = new GameObject();
        body.transform.SetParent(pivot.transform);
        body.AddComponent<SpriteRenderer>().sprite = _sprite;

        var pivotOffsetY = _sprite.rect.height / (2f * _sprite.pixelsPerUnit);
        body.transform.localPosition = new Vector3(0, pivotOffsetY, 0);
        pivot.transform.localPosition = new Vector3(0, -pivotOffsetY, 0);

        _body = body.transform;
        _nullBodyPos = body.transform.localPosition;
    }

	protected void Update()
	{
		var frequency = (_animTime += Time.deltaTime) * 18;
		_body.transform.localRotation = Quaternion.Euler(new Vector3(
			0,
			0,
			-_playbackRatio * Mathf.Sin(frequency) * 10) 
			);

		_body.transform.localPosition += new Vector3(0, Mathf.Abs(_playbackRatio) * Mathf.Sin(frequency) * 0.025f,0);
	}

	public void SetAnimRatio(float ratio, bool playbackFromStart = false)
	{
		if (Math.Abs(_playbackRatio - ratio) < Double.Epsilon)
			return;

		_playbackRatio = ratio;
		if (playbackFromStart)
		{
			_animTime = 0f;
			_body.localPosition = _nullBodyPos;
		}
	}

}

