using System;
using System.Linq;
using UnityEngine;

public static class ComponentFluentExtensions
{
	public static bool CheckComponents(this GameObject go, params Type[] components)
	{
		return components.All(component => go.GetComponent(component) != null);
	}

	public static GameObject Create(this GameObject go, Vector3 position, float degreeRotation = 0)
	{
		go.transform.position = position;
		go.transform.Rotate(Vector3.forward * degreeRotation);

		return go;
	}

	public static GameObject AddMovementComponent(this GameObject go, float maxSpeed)
	{
		MovementComponent.Create(go.gameObject, maxSpeed);
		return go;
	}

	public static GameObject AddCharacterComponent(this GameObject go)
	{
		CharacterComponent.Create(go.gameObject);
		return go;
	}

	public static GameObject AddPlayerComponent(this GameObject go, Camera camera)
	{
		PlayerComponent.Create(go.gameObject, camera);
		return go;
	}

	public static GameObject AddSimpleRendererComponent(this GameObject go, Sprite sprite)
	{
		SimpleRenderComponent.Create<SimpleRenderComponent>(go, sprite);
		return go;
	}

	public static GameObject AddCharacterRendererComponent(this GameObject go, Sprite sprite)
	{
		CharacterRenderComponent.Create(go, sprite);
		return go;
	}

	//public static GameObject AddIdeaHolderComponent(this GameObject go)
	//{
	//	IdeaHolderComponent.Create(go);
	//	return go;
	//}

}