﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using UnityEngine;


public class GDesignerController : ControllerComponent
{

	private float _theftTimer = 0f;
	private readonly float _theftCooldown = 2f;
	private Teampoint _theftTarget;
	//private bool _ideaToBack;

	private NavPoint _aim = null;
	private List<NavPoint> _path;

	private float _walkTimer = 1f;
	private float _wannaSeatTimer = 30f;
	private static float _walkPause = 4f;
	private static float _sittingPause = 7f;
	private static float _wannaSeatPause = 30f;
	//private static float _stayTablePause = 10f;
	private Chair _chair;
	private bool _aimToSeet = false;

	//private float _previousDistance = 100f;
	//private float _wrongDist = 100f;

	void Awake()
	{
		controlledObject = GetComponent<CharacterComponent>();
		movement = GetComponent<MovementComponent>();

		canControl = true;
		_path = new List<NavPoint>();
		_theftTarget = null;
		//_ideaToBack = false;
	}

	void Start()
	{
		movement.SetMovementSpeedRate(1);
		_wannaSeatTimer = UnityEngine.Random.Range(2, _wannaSeatPause);
	}

	void Update()
	{
		if (!canControl)
			return;

		if (Time.deltaTime > 0.5f)
		{
			SetMovementToAim();
		}

		if (_path.Count > 0)
		{
			if (_aim)
			{
				if (Vector2.Distance(_aim.transform.position, transform.position) < 0.15f)
				{
					_path.Remove(_aim);
					//Debug.Log("Path point remaining: " + _path.Count);
					if (_path.Count == 0)
					{
						SetMovementDirection(new Vector3(0, 0, 0));
						_aim = null;
						_walkTimer = _walkPause + UnityEngine.Random.Range(0, _walkPause / 2);

						if (_aimToSeet)
						{
							transform.position = _chair.transform.position;
							_chair.isWorking = true;
							_walkTimer = _sittingPause + UnityEngine.Random.Range(0, _sittingPause);

							// Bring ideas to my table
							var teamHolder = GetComponent<CharacterComponent>().GetTeam().GetComponent<IdeaHolderComponent>();
							var myHolder = GetComponent<IdeaHolderComponent>();
							foreach (var idea in myHolder.GetAllIdeas())
								teamHolder.AddIdea(idea);
							
						}
					}
					else
					{
						_aim = _path[0];
						SetMovementToAim();
						//Debug.Log("New point: " + _aim);
					}
				}
			}
			else
			{
				_aim = _path[0];
				Debug.LogError("NO AIM WHILE PATH COUNT IS ABOVE ZERO");
			}
		}
		else
		{
			if (_walkTimer < 0)
			{
				if (_chair)
				{
					_chair.isFree = true;
					_chair.isWorking = false;
					_chair = null;
					_wannaSeatTimer = _wannaSeatPause + UnityEngine.Random.Range(0, _wannaSeatPause / 2);
				}

				if (_theftTarget) { 
					var idea = _theftTarget.GetRandomIdea();
					if (idea)
					{
						controlledObject.SteelIdea(idea);
						_theftTimer = _theftCooldown + UnityEngine.Random.Range(0, _theftCooldown);
					}
					_theftTarget = null;
				}

				//if (_ideaToBack)
				//{
				//	var team = GetComponent<CharacterComponent>().GetTeam();
				//	if (Vector2.Distance(transform.position, team.transform.position) < 2f)
				//	{
				//		var holder = team.GetComponent<IdeaHolderComponent>();
				//		foreach (var idea in holder.GetAllIdeas())
				//			holder.AddIdea(idea);
				//	}
				//}

				NavPoint globalAim;
				//if (UnityEngine.Random.Range(0, 100) > 0 && GetComponent<IdeaHolderComponent>().GetActiveIdeasAmount() > 0)
				//{
				//	Debug.Log("go to my table");
				//	globalAim = GetNearestToTable(GetComponent<CharacterComponent>().GetTeam());
				//	_ideaToBack = true;
				//	_aimToSeet = false;
				//}
				//else
				//Debug.Log("Active: " + controlledObject.GetComponent<IdeaHolderComponent>().GetAllIdeas().Count);
				if (_wannaSeatTimer < 0 && GetComponent<IdeaHolderComponent>().GetAllIdeas().Count > 0 && (_chair = GetComponent<CharacterComponent>().GetTeam().GetFreeChair()))
				{
					globalAim = NearestNavPoint(_chair.transform);
					_aimToSeet = true;
					//Debug.Log("go sit");
				}
				else if (_theftTimer <= 0)
				{
					globalAim = RandomTeamTable(out _theftTarget);
					_aimToSeet = false;
				}
				else
				{
					globalAim = NavMaster.instance.GetRandomPoint();
					_aimToSeet = false;
					//Debug.Log("go go");
				}

				_aim = NearestNavPoint(transform);
				_path = NavMaster.instance.GetPathFromTo(_aim, globalAim);

				SetMovementToAim();

				//Debug.Log("Global: " + globalAim);
			}

		}

		_walkTimer -= Time.deltaTime;
		_wannaSeatTimer -= Time.deltaTime;
		_theftTimer -= Time.deltaTime;
	}

	private NavPoint NearestNavPoint(Transform from)
	{
		var navPoints = GameObject.FindObjectsOfType<NavPoint>();
		NavPoint nearest = navPoints[0];
		float minDistance = 100f;

		for (var i = 1; i < navPoints.Length; i++)
		{
			var navPoint = navPoints[i];
			var dist = Vector2.Distance(from.position, navPoint.transform.position);
			if (dist < minDistance)
			{
				minDistance = dist;
				nearest = navPoint;
			}
		}

		return nearest;
	}

	private void SetMovementToAim()
	{
		var angle = Mathf.Rad2Deg * Mathf.Atan2(_aim.transform.position.y - transform.position.y, _aim.transform.position.x - transform.position.x);
		SetMovementDirection(new Vector3(Mathf.Cos(angle * Mathf.Deg2Rad), Mathf.Sin(angle * Mathf.Deg2Rad), 0));
	}

	private NavPoint RandomTeamTable(out Teampoint tp)
	{
		var tps = GameObject.FindObjectsOfType<Teampoint>().Where(t => t != GetComponent<CharacterComponent>().GetTeam()).ToList();
		tp = tps[UnityEngine.Random.Range(0, tps.Count)];
		return GetNearestToTable(tp);
	}

	private NavPoint GetNearestToTable(Teampoint tp)
	{
		var navPoints = GameObject.FindObjectsOfType<NavPoint>();
		var nearest = navPoints.Where(navPoint => Vector2.Distance(tp.transform.position, navPoint.transform.position) < 1.4f).ToList();

		return nearest[UnityEngine.Random.Range(0, nearest.Count)];
	}

	public void StopActions()
	{
		SetMovementDirection(new Vector3(0, 0, 0));
		canControl = false;
		_aim = null;
		_path.Clear();
		_theftTarget = null;
	}

	public void GetBackActions()
	{
		canControl = true;
	}

}

