﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using UnityEngine;


public class IdeaHolderComponent : MonoBehaviour
{
	//public bool isFreeSlot
	//{
	//	get { return GetRandomIdeaSlot() != null; }
	//}

	public static IdeaHolderComponent Create(GameObject go)
	{
		return go.AddComponent<IdeaHolderComponent>();
	}

	public bool hideAll
	{
		set
		{
			foreach (Idea idea in _ideasSlots)
			{
				if (idea != null)
					idea.SetHiding(value);
			}
		}
	}

	public int ideasCount { get; private set; }

	private Idea[,] _ideasSlots = new Idea[3, 3];

	public Idea CreateIdea(bool hiding)
	{
		var img = GameManager.instance.GetIdeaImage();
		if (img == null)
			return null;

		var randSlot = GetRandomIdeaSlot();
		if (randSlot == null) return null;

		var idea = Idea.Create(GameManager.instance.ideaPrefab, img, transform, randSlot.Value, hiding);
		_ideasSlots[(int)(randSlot.Value.x + 1), (int)(randSlot.Value.y + 1)] = idea;
		ideasCount++;
		return idea;
	}

	public bool AddIdea(Idea newIdea)
	{
		if (newIdea == null)
		{
			//Debug.Log("idea is null");
			return false;
		}

		var randSlot = GetRandomIdeaSlot();
		if (randSlot == null || newIdea.isHided)
		{
			//Debug.Log("But already hided!");
			return false;
		}

		newIdea.ChangeIdeaParent(transform, randSlot.Value);
		_ideasSlots[(int)(randSlot.Value.x + 1), (int)(randSlot.Value.y + 1)] = newIdea;
		ideasCount++;

		return true;
	}

	public void RemoveIdea(Idea idea)
	{
		if (_ideasSlots[0, 0] == idea)
			_ideasSlots[0, 0] = null;
		else if (_ideasSlots[0, 1] == idea)
			_ideasSlots[0, 1] = null;
		else if (_ideasSlots[0, 2] == idea)
			_ideasSlots[0, 2] = null;
		else if (_ideasSlots[1, 0] == idea)
			_ideasSlots[1, 0] = null;

		// no 1-1 - no idea in center

		else if (_ideasSlots[1, 2] == idea)
			_ideasSlots[1, 2] = null;
		else if (_ideasSlots[2, 0] == idea)
			_ideasSlots[2, 0] = null;
		else if (_ideasSlots[2, 1] == idea)
			_ideasSlots[2, 1] = null;
		else if (_ideasSlots[2, 2] == idea)
			_ideasSlots[2, 2] = null;

		ideasCount--;
	}

	private Vector2? GetRandomIdeaSlot()
	{
		var freeSlots = new List<Vector2>();
		if (_ideasSlots[0, 0] == null)
			freeSlots.Add(new Vector2(-1, -1));
		if (_ideasSlots[0, 1] == null)
			freeSlots.Add(new Vector2(-1, 0));
		if (_ideasSlots[0, 2] == null)
			freeSlots.Add(new Vector2(-1, 1));
		//if (_ideasSlots[1, 0] == null)
		//	freeSlots.Add(new Vector2(0, -1));

		// no 1-1 - no idea in center

		if (_ideasSlots[1, 2] == null)
			freeSlots.Add(new Vector2(0, 1));
		//if (_ideasSlots[2, 0] == null)
		//	freeSlots.Add(new Vector2(1, -1));
		if (_ideasSlots[2, 1] == null)
			freeSlots.Add(new Vector2(1, 0));
		if (_ideasSlots[2, 2] == null)
			freeSlots.Add(new Vector2(1, 1));

		if (freeSlots.Count > 0)
		{
			return freeSlots[UnityEngine.Random.Range(0, freeSlots.Count)];
		}
		else
		{
			return null;
		}
	}

	public List<Idea> GetAllIdeas()
	{
		return _ideasSlots.Cast<Idea>().Where(idea => idea != null).ToList();
	}

	public int GetActiveIdeasAmount()
	{
		return _ideasSlots.Cast<Idea>().Count(idea => idea != null && !idea.isHided);
	}

	public Idea GetRandomIdea()
	{
		var ideas = _ideasSlots.Cast<Idea>().Where(idea => idea != null).ToList();
		return 
			ideas.Count == 0 
			? null 
			: ideas[UnityEngine.Random.Range(0, ideas.Count)];
	}

	internal void RemoveIdea()
	{
		var idea = _ideasSlots.Cast<Idea>().FirstOrDefault(i => i != null);
		if (idea != null) Destroy(idea.gameObject);
	}
}

