﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using UnityEngine;


public class PersonController : ControllerComponent
{

	//private float _theftTimer = 0f;
	//private readonly float _theftCooldown = 5f;

	private NavPoint _aim = null;
	private List<NavPoint> _path;

	private float _walkTimer = 1f;
	private float _wannaSeatTimer = 2f;
	private static float _walkPause = 3f;
	private static float _sittingPause = 25f;
	private static float _wannaSeatPause = 15f;
	private Chair _chair;
	private bool _aimToSeet = false;

	//private float _previousDistance = 100f;
	//private float _wrongDist = 100f;

	void Awake()
	{
		controlledObject = GetComponent<CharacterComponent>();
		movement = GetComponent<MovementComponent>();

		canControl = true;
		_path = new List<NavPoint>();
	}

	void Start()
	{
		movement.SetMovementSpeedRate(1);
		_wannaSeatTimer = UnityEngine.Random.Range(2, 12);
	}

	void Update()
	{
		if (!canControl)
			return;

		if (Time.deltaTime > 0.5f)
		{
			SetMovementToAim();
		}

		if (_path.Count > 0)
		{
			if (_aim)
			{
				if (Vector2.Distance(_aim.transform.position, transform.position) < 0.15f)
				{
					_path.Remove(_aim);
					//Debug.Log("Path point remaining: " + _path.Count);
					if (_path.Count == 0)
					{
						SetMovementDirection(new Vector3(0, 0, 0));
						_aim = null;
						_walkTimer = _walkPause + UnityEngine.Random.Range(0, _walkPause / 2);

						if (_aimToSeet)
						{
							transform.position = _chair.transform.position;
							_chair.isWorking = true;
							_walkTimer = _sittingPause + UnityEngine.Random.Range(0, _sittingPause);
						}
					}
					else
					{
						_aim = _path[0];
						SetMovementToAim();
						//Debug.Log("New point: " + _aim);
					}
				}
			}
			else
			{
				_aim = _path[0];
				Debug.LogError("NO AIM WHILE PATH COUNT IS ABOVE ZERO");
			}
		}
		else
		{
			if (_walkTimer < 0)
			{
				if (_chair)
				{
					_chair.isFree = true;
					_chair.isWorking = false;
					_chair = null;
					_wannaSeatTimer = _wannaSeatPause + UnityEngine.Random.Range(0, _wannaSeatPause / 2);
				}

				NavPoint globalAim;
				if (_wannaSeatTimer < 0 && (_chair = GetComponent<CharacterComponent>().GetTeam().GetFreeChair()))
				{
					globalAim = NearestNavPoint(_chair.transform);
					_aimToSeet = true;
					//Debug.Log("go sit");
				}
				else
				{
					globalAim = NavMaster.instance.GetRandomPoint();
					_aimToSeet = false;
					//Debug.Log("go go");
				}

				_aim = NearestNavPoint(transform);
				_path = NavMaster.instance.GetPathFromTo(_aim, globalAim);

				SetMovementToAim();

				//Debug.Log("Global: " + globalAim);
			}

		}

		_walkTimer -= Time.deltaTime;
		_wannaSeatTimer -= Time.deltaTime;
	}

	private NavPoint NearestNavPoint(Transform from)
	{
		var navPoints = GameObject.FindObjectsOfType<NavPoint>();
		NavPoint nearest = navPoints[0];
		float minDistance = 100f;

		for (var i = 1; i < navPoints.Length; i++)
		{
			var navPoint = navPoints[i];
			var dist = Vector2.Distance(from.position, navPoint.transform.position);
			if (dist < minDistance)
			{
				minDistance = dist;
				nearest = navPoint;
			}
		}

		return nearest;
	}

	private void SetMovementToAim()
	{
		var angle = Mathf.Rad2Deg * Mathf.Atan2(_aim.transform.position.y - transform.position.y, _aim.transform.position.x - transform.position.x);
		SetMovementDirection(new Vector3(Mathf.Cos(angle * Mathf.Deg2Rad), Mathf.Sin(angle * Mathf.Deg2Rad), 0));
	}

	//private NavPoint NextPathPoint()
	//{
	//	float angle;
	//	var newAim = _aim.GetBestWayPointTo(_globalAim, transform, out angle);

	//	//var angle = Mathf.Rad2Deg * Mathf.Atan2(_aim.transform.position.y - transform.position.y, _aim.transform.position.x - transform.position.x);
	//	SetMovementDirection(new Vector3(Mathf.Cos(angle * Mathf.Deg2Rad), Mathf.Sin(angle * Mathf.Deg2Rad), 0));
	//	return newAim;
	//}

	//private void SetMovement(float angle)
	//{
	//	SetMovementDirection(new Vector3(Mathf.Cos(angle * Mathf.Deg2Rad), Mathf.Sin(angle * Mathf.Deg2Rad), 0));
	//}

	//private void SetGlobalPath(NavPoint destination)
	//{
	//	_globalAim = destination;
	//}

}

