﻿using System;
using System.Collections.Generic;
using System.Linq;
using UnityEngine;

public class PlayerComponent : ControllerComponent
{
    [SerializeField]
	private Camera _camera;

	[SerializeField] private GameObject _timerCooldown;

	private Vector3 _aim;

	private GDesignerController _tuningDesigner;

	private int cameraSpeed = 15;
	bool left = false, up = false, down = false, right = false;
	bool mouseLeft = false, mouseUp = false, mouseDown = false, mouseRight = false;

	private float _theftTimer = 0f;
	private readonly float _theftCooldown = 5f;

	public static PlayerComponent Create(GameObject go, Camera camera)
	{
		if (!go.CheckComponents(typeof(CharacterComponent)))
		{
			Debug.LogError("Can't create PlayerComponent for object without CharacterComponent");
			return null;
		}

	    var pc = go.AddComponent<PlayerComponent>();
		pc._camera = camera;
		
		return pc;
	}

    void Awake()
    {
        controlledObject = GetComponent<CharacterComponent>();
        movement = GetComponent<MovementComponent>();
        
        _aim = transform.position;
        canControl = true;
    }

    void Start()
    {
        movement.SetMovementSpeedRate(1);
    }

	protected override void Update()
	{
		if (!canControl)
			return;

		// Mouse control
		if (Input.GetMouseButtonDown(1))
		{
			_aim = _camera.ScreenToWorldPoint(Input.mousePosition);
			_aim.z = 0;
		}
		
		if (Vector2.Distance(_aim, transform.position) < 0.2f)
		{
			SetMovementDirection(new Vector3(0, 0, 0));
		}
		else
		{
			var angle = Mathf.Rad2Deg * Mathf.Atan2(_aim.y - transform.position.y, _aim.x - transform.position.x);
			SetMovementDirection(new Vector3(Mathf.Cos(angle * Mathf.Deg2Rad), Mathf.Sin(angle * Mathf.Deg2Rad), 0));
		}

		CameraControls();

		_theftTimer -= Time.deltaTime;
		if (_theftTimer < 0 && _timerCooldown.activeSelf)
		{
			_timerCooldown.SetActive(false);
		}
		if (Input.GetMouseButtonDown(0))
		{
			var hits = Physics2D.RaycastAll(Camera.main.ScreenToWorldPoint(Input.mousePosition), Vector2.zero, 1000);
			if (hits != null)
			{
				var idea = hits.Where(h => h.collider.transform.gameObject.GetComponent<Idea>() != null)
					.Select(h => h.collider.transform.gameObject.GetComponent<Idea>())
					.FirstOrDefault();
				
			    if (idea != null)
			    {
			        if (idea.IsItMyIdea(controlledObject))
			        {
			            var team = controlledObject.GetTeam();
			            //Debug.Log(team);
			            if (Vector2.Distance(team.transform.position, transform.position) < 1.8f)
			            {
			                team.GetComponent<IdeaHolderComponent>().AddIdea(idea);
			            }
			        }
			        else {
						if (_theftTimer <= 0 && Vector2.Distance(idea.GetOwnerPos(), transform.position) < 1.7f)
						{
							if (idea.CanSteel(controlledObject))
							{
								//Debug.Log("Steeling");
								controlledObject.SteelIdea(idea);

								
							}
							else
							{
								//Debug.Log("cAn't steel");
							}
							if (!idea.isHided)
							{
								_theftTimer = _theftCooldown;
								_timerCooldown.SetActive(true);
							}
						}
						
					}
			        
			    }
			    else
			    {
				    var gd = hits
						.Where(h => h.collider.transform.gameObject.GetComponent<GDesignerController>() != null)
						.Select(h => h.collider.transform.gameObject.GetComponent<GDesignerController>())
						.FirstOrDefault();

				    if (gd != null 
						&& _theftTimer <= 0
						&& gd.GetComponent<IdeaHolderComponent>().GetAllIdeas().Count > 0
						&& Vector2.Distance(gd.transform.position, transform.position) < 0.99f)
				    {
						Debug.Log("StartTuning");
					    _tuningDesigner = gd;
						gd.StopActions();
					    canControl = false;
						SetMovementDirection(new Vector3(0, 0, 0));
						GameManager.instance.waveTuner.gameObject.SetActive(true);
						GameManager.instance.waveTuner.Init(OnTuneSuccess, OnStopTune);
				    }
			    }

				
			}
		}
		
	}

	private void OnTuneSuccess()
	{
		if (_tuningDesigner)
		{
			_tuningDesigner.GetComponent<IdeaHolderComponent>().RemoveIdea();
		}
	}

	private void OnStopTune()
	{
		if (_tuningDesigner)
		{
			_tuningDesigner.GetBackActions();
			_tuningDesigner = null;
		}
		GetComponent<IdeaHolderComponent>().RemoveIdea();
		canControl = true;
		_theftTimer = _theftCooldown;
		_timerCooldown.SetActive(true);
	}

	private void CameraControls()
	{
		// Camera controls
		Vector3 speed = new Vector3();
		//if (Input.GetKeyDown(KeyCode.A))
		//{
		//	left = true;
		//}
		//if (Input.GetKeyDown(KeyCode.W))
		//{
		//	up = true;
		//}
		//if (Input.GetKeyDown(KeyCode.D))
		//{
		//	right = true;
		//}
		//if (Input.GetKeyDown(KeyCode.S))
		//{
		//	down = true;
		//}

		//if (Input.GetKeyUp(KeyCode.A))
		//{
		//	left = false;
		//}
		//if (Input.GetKeyUp(KeyCode.W))
		//{
		//	up = false;
		//}
		//if (Input.GetKeyUp(KeyCode.D))
		//{
		//	right = false;
		//}
		//if (Input.GetKeyUp(KeyCode.S))
		//{
		//	down = false;
		//}

		if (Input.mousePosition.x < 15)
			mouseLeft = true;
		else
			mouseLeft = false;

		if (Input.mousePosition.y > Screen.height - 15)
			mouseUp = true;
		else
			mouseUp = false;

		if (Input.mousePosition.x > Screen.width - 15)
			mouseRight = true;
		else
			mouseRight = false;

		if (Input.mousePosition.y < 15)
			mouseDown = true;
		else
			mouseDown = false;

		if (left || mouseLeft)
			speed += new Vector3(-cameraSpeed, 0, 0);
		if (up || mouseUp)
			speed += new Vector3(0, cameraSpeed, 0);
		if (right || mouseRight)
			speed += new Vector3(cameraSpeed, 0, 0);
		if (down || mouseDown)
			speed += new Vector3(0, -cameraSpeed, 0);

		var newCamPos = _camera.transform.position + speed * Time.deltaTime;
        
        // Clamp camera position
	    newCamPos.x = Mathf.Clamp(newCamPos.x, -1.8f, 1.5f);
	    newCamPos.y = Mathf.Clamp(newCamPos.y, -2f, 0.6f);
	    _camera.transform.position = newCamPos;
	}

}