﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using UnityEngine;

public class SimpleRenderComponent : MonoBehaviour
{
	// Only use for flip rotation
	protected Transform _body;
	protected Vector3 _nullBodyPos;

	public static T Create<T>(GameObject go, Sprite sprite)
		where T : SimpleRenderComponent
	{
		var ren = go.AddComponent<T>();

		var pivot = new GameObject();
		pivot.transform.SetParent(go.transform);

		var body = new GameObject();
		body.transform.SetParent(pivot.transform);
		body.AddComponent<SpriteRenderer>().sprite = sprite;

		var pivotOffsetY = sprite.rect.height / (2f * sprite.pixelsPerUnit);
		body.transform.localPosition = new Vector3(0, pivotOffsetY, 0);
		pivot.transform.localPosition = new Vector3(0, -pivotOffsetY, 0);

		ren._body = body.transform;
		ren._nullBodyPos = body.transform.localPosition;

		return ren;
	}

	public void SetHorizontalFlip(sbyte rightDirection)
	{
		if (rightDirection == 0)
			return;
		
		_body.localRotation = Quaternion.Euler(0, 90 * (1 - rightDirection), 0);
	}

	public void SetVerticalFlip(sbyte topDirection)
	{ 
		if (topDirection == 0)
			return;

		_body.localRotation = Quaternion.Euler(90 * (1 - topDirection), 0, 0);
	}

}

